import time
import fire
from . import tail, sender


def agal(*args, uri='', key=''):
    if uri:
        msg_sender = sender.Sender(uri, key=key)
        async def line_handler(line, **kwargs):
            await msg_sender.send({
                'payload': line,
                'meta': {
                    'timestamp': int(time.time())
                }
            })
    else:
        line_handler = None
    tail.start(args, handler=line_handler)


def main():
    fire.Fire(agal)


if __name__ == '__main__':
    main()



import json
import random
import hashlib
import base64
from cryptography.fernet import Fernet, InvalidToken
from . import constant


def gen_key():
    with open('%s/%s' % (constant.LOCATION, 'hamlet.json')) as fd:
        hf_key = '-'.join(random.sample(json.loads(fd.read()), 4))
        return hf_key


def to_b64_key(hf_key):
    b64_key = base64.urlsafe_b64encode(hashlib.sha256(hf_key.encode('utf-8')).digest())
    return b64_key


class MessageCrypt:
    def __init__(self, key):
        self.fernet = Fernet(to_b64_key(key))

    def json_encrypt(self, data):
        data = json.dumps(data).encode('utf-8')
        return self.fernet.encrypt(data)

    def json_decrypt(self, token):
        try:
            if not isinstance(token, bytes):
                token = token.encode('utf-8')
            data = json.loads(self.fernet.decrypt(token))
            return data
        except InvalidToken:
            return None



import sys
import os
import time
import asyncio
import collections
import threading
import concurrent.futures

pool = concurrent.futures.ThreadPoolExecutor()

SEP = '\n'
LINES = collections.deque()


def fd_list_to_dict(fd_list):
    fd_dict = {}
    for idx, fd in enumerate(fd_list):
        fd_dict[idx] = {
            'fd': fd,
            'buf': '',
            'idx': idx
        }
    return fd_dict


async def readlines(fd_info):
    global LINES
    loop = asyncio.get_running_loop()
    line = await loop.run_in_executor(pool, fd_info['fd'].readline)
    if not line:
        await asyncio.sleep(0.1)
        return
    LINES.append(line)


async def tail(fd_list):
    fd_dict = fd_list_to_dict(fd_list)
    while True:
        for idx, fd_info in fd_dict.items():
            await readlines(fd_info)


async def handle_lines(handler=None):
    global LINES
    while True:
        try:
            line = LINES.popleft()
            if handler:
                await handler(line)
            else:
                sys.stdout.write(line)
        except:
            await asyncio.sleep(0.1)


def start_loop(futures):
    def _start_loop():
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.run_until_complete(futures)
        loop.close()
    t = threading.Thread(target=_start_loop, daemon=True)
    t.start()
    return t


def start(filename_list, handler=None):
    fd_list = []
    for i in filename_list:
        if os.path.exists(i):
            fd = open(i)
            fd.seek(0, 2)
            fd_list.append(fd)
    if not sys.stdin.isatty():
        fd_list.append(sys.stdin)
    t_read = start_loop(tail(fd_list))
    t_handle = start_loop(handle_lines(handler))
    t_read.join()
    t_handle.join()



import os.path
import asyncio
import websockets
import threading
import http
import threading
import base64
import json
import collections
import mimetypes
import fire
import urllib.parse
from . import message_crypt
from . import constant


class Global:
   users = set()
   messages = collections.deque()
   message_count = 0
   config = {}


async def main_input(host, port, key):
    crypt = None
    if key:
        crypt = message_crypt.MessageCrypt(key)
    async def recv(websocket):
        async for entry in websocket:
            if crypt:
                message = crypt.json_decrypt(entry)
            else:
                message = entry
            print("Sending %s" % message)
            if message:
                Global.messages.append(message)

    async with websockets.serve(
        recv, host, port
    ):
        await asyncio.Future()  # run forever


async def static_serve(path, request_headers):
    parsed = urllib.parse.urlparse(path)
    if Global.config.get('username') and Global.config.get('password'):
        response_header = [('WWW-Authenticate', 'Basic realm="Enter the credentials to access."')]
        auth_header = request_headers.get('Authorization')
        try:
            realm, credentials = auth_header.split(' ')
            if realm == 'Basic':
                username, password = base64.b64decode(credentials.encode('utf-8')).decode('utf-8').split(':')
                if username == Global.config['username'] and password == (Global.config['password']):
                    pass
                else:
                    response_header = []
                    raise Exception()
        except Exception as e:
            return (http.HTTPStatus.UNAUTHORIZED, response_header, b'')
    if parsed.path == '/':
        file_path = '%s/static/index.html' % constant.LOCATION
    else:
        file_path = '%s/static/%s' % (constant.LOCATION,
                                      parsed.path.replace('..', ''))
    if os.path.exists(file_path):
        content_type = mimetypes.guess_type(file_path)[0] or 'text/plain'
        return (http.HTTPStatus.OK,
                [('Content-Type', content_type)],
                open(file_path, 'rb').read())


async def distribution(websocket):
   try:
       Global.users.add(websocket)
       while True:
           try:
               message = Global.messages.popleft()
               Global.message_count += 1
               if isinstance(message, dict) and message.get('meta'):
                   message['meta']['key'] = '%s%010d' % (message['meta'].get('timestamp', ''),
                                                        Global.message_count)
               websockets.broadcast(Global.users, json.dumps(message))
           except Exception as e:
               if isinstance(e, IndexError):
                   pass
               else:
                   print(e)
               await asyncio.sleep(0.1)
   finally:
       Global.users.remove(websocket)


async def main_web(host, port):
    async with websockets.serve(
        distribution, host, port,
        process_request=static_serve,
    ):
        await asyncio.Future()


def start_web(*args):
    loop  = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_until_complete(main_web(*args))
    loop.close()


def start_input(*args):
    loop  = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_until_complete(main_input(*args))
    loop.close()


def serve(key="", plain=False,
          host='0.0.0.0', port='8800',
          input_host='127.0.0.1', input_port='8700',
          username='', password=''):
    if not (key or plain):
        key = message_crypt.gen_key()
        print("Encryption key generated:\n%s\n" % key)
    if username and password:
        Global.config['username'] = username
        Global.config['password'] = password
    thread_web = threading.Thread(target=start_web,
                                  args=(host, int(port)),
                                  daemon=True)
    thread_input = threading.Thread(target=start_input,
                                    args=(input_host, int(input_port), key),
                                    daemon=True)
    thread_web.start()
    thread_input.start()
    thread_web.join()
    thread_input.join()


def main():
    fire.Fire(serve)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['agal']

package_data = \
{'': ['*']}

install_requires = \
['cryptography>=3.4.7,<4.0.0', 'fire>=0.3.1,<0.4.0', 'websockets>=10.1,<11.0']

entry_points = \
{'console_scripts': ['agal-cli = agal.cli:main',
                     'agal-server = agal.server:main']}

setup_kwargs = {
    'name': 'agal',
    'version': '0.0.3',
    'description': '',
    'long_description': None,
    'author': 'Gongziting Tech Ltd.',
    'author_email': None,
    'url': 'https://github.com/gzttech/agal',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import websockets
from . import message_crypt


class Sender:
    def __init__(self, uri, key=None):
        self.uri = uri
        self.key = key
        self.conn = None
        if self.key:
            self.crypt = message_crypt.MessageCrypt(self.key)

    async def connect(self):
        if (not self.conn) or self.conn.closed:
            self.conn = await websockets.connect(self.uri)

    async def send(self, message):
        await self.connect()
        send_message = message if not self.key\
            else self.crypt.json_encrypt(message)
        await self.conn.send(send_message)
